/*
			 Salvatore Ruggieri (c), 2002-2008
*/

/** @file util.h  */

#ifndef _UTIL_H__INCLUDED_
#define _UTIL_H__INCLUDED_

#if !defined(__GNUC__)
	#error "Unknown compiler."
#endif

extern "C" {
#include <polylib/polylib.h>
}

#include <cmath>
#include <cstddef>
#include <cstdio>
#include <cstdlib>
#include <ctime>
#include <fstream> /* standard fstream */
#include <inttypes.h>
#include <iostream> /* standard iostream */
#include <limits>
#include <stdexcept>
#include <string>
#include <sys/time.h>
#include <sys/times.h>
#include <vector>
#ifndef DOXYGEN_SHOULD_SKIP_THIS
using namespace std;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

/** 64 bit signed integer integral data type. */
typedef long long int64;
/** 64 bit unsigned integer integral data type. */
typedef unsigned long long uint64;

/** Utility classes.
  * The using directive is automatically included in the header <basic.h>
  */
namespace util
{

/** Global %verbose level. */
class verbose
{
public:
	static int level;
};

/** Global %unique identifier. */
class unique
{
public:
	static uint64 id;
};

/**
* Base class to prevent copy and assignment operator.
*/
class nocopy
{
protected:
	/** Constructor. */
	nocopy() {}
	/** Destructor. Not virtual! */
	~nocopy() {}
private:
	/** Copy constructor private and not defined. */
	nocopy( const nocopy& );
	/** Assignment constructor private and not defined. */
	const nocopy& operator=( const nocopy& );
};

/**
* Extracts tokens separated by a char from a buffer. The buffer
* is a const C string, i.e. a const char*.
*/
class tokenizer : nocopy
{
public:
	/** Constructor from an unmodifiable C string.
			@param buffer a const C string.
			@param delim token separator.*/
	tokenizer(const char * buffer = NULL, char delim = '\t')
	{
		const size_t initial_length = 256;
		last_token = new char[initial_length];
		if( last_token == NULL )
			throw runtime_error("Out of memory");
		last_token_end = last_token+initial_length;

		tokenizing(buffer, delim);
	}
	/**  Destructor. */
	virtual ~tokenizer()
	{
		delete []last_token;
	}
	/** Start tokenizing from an unmodifiable C string.
			@param buffer a const C string.
			@param delim token separator. */
	void tokenizing(const char * buffer = NULL, char delim = '\t')
	{
		del = delim;
		buffer_start =
		buf = (buffer == NULL) ? &EMPTY : buffer;
	}
	/** Return the number of tokens still to be extracted from buffer. */
	size_t count() const
	{
		const char *p = buf;
		size_t ntokens = 1;

		while( *p ) {
			if( (*p++ == del) && (*p) )
				 ++ntokens;
		}

		return ntokens;
	}
	/** Return true if there is at least one more token to be extracted from buffer. */
	bool has_next() const
	{
		return (*buf);
	}
	/** Extract next token and return a const pointer to it. The token is stored
	locally and it is valid until next call to next().
		@returns NULL if there is no more token. */
	const char* next()
	{
		return next(this->del);
	}
	/** Extract next token and return a const pointer to it. The token is stored
	locally and it is valid until next call to next().
	    @param del token delimiter
		@returns NULL if there is no more token. */
	const char* next(char del)
	{
		const char *p = buf;
		char *temp = last_token;

		while( (*p != del) && (*temp++ = *p) ) {
			++p;
			if( temp == last_token_end ) {
				// enlarge token length
				size_t token_len = last_token_end - last_token;
				size_t new_token_len = token_len + token_len/2;
				char *new_last_token = new char[new_token_len];
				if( new_last_token == NULL )
					throw runtime_error("Out of memory");
				last_token_end = new_last_token + new_token_len;
				memcpy(new_last_token, last_token, token_len);
				delete []last_token;
				last_token = new_last_token;
				temp = last_token+token_len;
			}
		}

		if( *p )
		{
			*temp = '\0';
			buf = p + 1;
		}
		else
		{
			buf = p;
		}
		return last_token;
	}
	/** Return a const char pointer to the rest of buffer still to be tokenized. The
	buffer is stored locally and it is valid until next call to next_token().*/
	const char* get_remainder() const
	{
		return buf;
	}
	/** Return a const pointer to the whole string that is being tokenized. */
	const char* get_original() const
	{
		return buffer_start;
	}
private:
	/** Pointer to the portion of buffer still to tokenize. */
	const char *buf;
	/** Pointer to beginning of buffer. */
	const char *buffer_start;
	/**  Last extracted token. */
	char *last_token;
	/**  End of last extracted token. */
	char *last_token_end;
	/** Token separator. */
	char del;
	/** An empty buffer. */
	static const char EMPTY;
};


/**
* Static utitity functions involving strings.
*/
class string_util
{
public:

	/** Remove all spaces in a string. */
	static char *remove_spaces(const char *s, const char *e = NULL)
	{
		size_t len = 0;
		const char *start = s;
		if( e == NULL )
			while( *start ) {
				if( *start == ' ' || *start == '\n' || *start == '\t' || *start == '\r') {
					++start;
					continue;
				}
				++len;
				++start;
			}
		else
			while( *start && (start < e)) {
				if( *start == ' ' || *start == '\n' || *start == '\t' || *start == '\r') {
					++start;
					continue;
				}
				++len;
				++start;
			}
		e = start;
		char *ret = new char[len+1];
		char *r = ret;
		start = s;
		while( start < e ) {
 			if( *start == ' ' || *start == '\n' || *start == '\t' || *start == '\r') {
				++start;
				continue;
			}
			*r++ = *start++;
		}
		*r = 0;
		return ret;
	}

	/** Find a char separator. */
	static const char *find_sep(const char *start, char sep)
	{
		while( *start )
			if(*start == sep)
				return start;
			else
				++start;
		return start;

	}

	/** Find one out of two char separators. */
	static const char *find_sep2(const char *start, char sep1, char sep2)
	{
		while( *start )
			if( (*start == sep1) || (*start == sep2))
				return start;
			else
				++start;
		return start;

	}

	/** Find one out of three char separators. */
	static const char *find_sep3(const char *start, char sep1, char sep2, char sep3)
	{
		while( *start )
			if( (*start == sep1) || (*start == sep2)|| (*start == sep3))
				return start;
			else
				++start;
		return start;

	}

	/** True if parsing a number. */
	static bool is_number(const char *s, double *res)
	{
		char *ptr;
		double r = strtod(s, &ptr);
		if( *ptr )
			return false;
		if( res != NULL )
			*res = r;
		return true;
	}

	/**
	* A version of %strcpy. It differs from strncpy in allocating the
	* needed memory.
	* @param s source C string.
	* @return a newly allocated copy of the source string or NULL if
	*           allocation failed. */
	static char *strcpya(const char *s)
	{
		size_t len = strlen(s) + 1;

		char *copy = new char[len];
		if( copy == NULL )
			return NULL;
		return (char *)memcpy(copy, s, len);
	}

	/**
	* A version of %strcpy. It differs from strncpy in allocating the
	* needed memory.
	* @param s source C string.
	* @param n max characters.
	* @return a newly allocated copy of the source string or NULL if
	*           allocation failed. */
	static char *strcpya(const char *s, size_t n)
	{
		size_t len = n + 1;

		char *copy = new char[len];
		if( copy == NULL )
			return NULL;
		memcpy(copy, s, n);
		copy[n] = 0;
		return copy;
	}

	/**
	* Remove left and right spaces. */
	static char *trim(const char *s)
	{
		while( *s && (*s == ' '))
			++s;

		const char *start = s;
		const char *end = s;

		while( *++s )
			if( *s != ' ' )
				end = s;

		return strcpya(start, end - start + 1);
	}

	/**
	* Return a string representation of a number. */
	static string tostr(double f)
	{
		char c[256];

		int n = sprintf(c, "%.30f", f);
		char *s = c + n - 1;
		while( *s == '0' )
			--s;
		if( *s == '.' )
			*s = '\0';
		else
			*(s+1) = '\0';

		return c;
	}

	/**
	* Return power(10,number of decimal positions). */
	static double decimals(double d)
	{
		double n = 1;
		if( d < 0 ) d = -d;
		double f;
		while( ::modf(d,&f) != 0) {
			d *= 10;
			n *= 10;
		}
		return n;
	}

	};

/**
* Models a time cronometer.
*/
class cronos
{
public:
	/** Constructor. */
	cronos()
	: elapsedTime(0)
	{
	}
	/** Destructor. */
	virtual ~cronos()
	{
	}
	/** Start cronometer. */
	void start()
	{
		gettimeofday(&startTimeElapsed, NULL);
	}
	/** Stop cronometer. */
	void stop()
	{
		struct timeval endTimeElapsed;
		gettimeofday(&endTimeElapsed, NULL);
		int64 msecs = (int64(endTimeElapsed.tv_sec)*1000000+endTimeElapsed.tv_usec)
		  - (int64(startTimeElapsed.tv_sec)*1000000+startTimeElapsed.tv_usec);
		elapsedTime = msecs / (double)1000000;
	}
	/** Get elapsed time (in seconds) between last %pair of start(), stop() calls. */
	double get_elapsed() const
	{
		return elapsedTime;
	}

private:
	/** Holds elapsed time. */
	double elapsedTime;
	/** Holds start time. */
	struct timeval startTimeElapsed;
};

/** Models a linear system Ax <= b. */
class LinearSystem : nocopy
{
public:

	/** Name of constant value column. */
	#define CONSTCOLNAME "const"

	/** Row types: equality, strict inequality, inequality. */
	typedef enum {MAT_EQ = 0, MAT_LT, MAT_LE} RowType;

	/** Constructor. */
	LinearSystem(const string &constraints = "")
		: n_rows(0), n_cols(0), frozen(false)
	{
		add_col(CONSTCOLNAME);
		add(constraints);
	}

	/** Destructor. */
	virtual ~LinearSystem()
	{
		for(size_t i=0;i<n_cols;++i)
			delete[] matcols[i];
	}

	/** Add new constraints. */
	void add(const string &constraints)
	{
		if(constraints == "" )
			return;
		tokenizer tk(constraints.c_str(), ',');
		size_t crow = n_rows;
		add_rows(tk.count());
		while( tk.has_next() )
			parse(tk.next(), crow++);
	}

	/** Number of rows. */
	size_t getNrows() const
	{
		return n_rows;
	}

	/** Number of columns. */
	size_t getNcols() const
	{
		return n_cols-1;
	}

	/** Column name. */
	const string &getColName(size_t i) const
	{
		return matcolnames[i];
	}

	/** Column name. */
	void renameColName(size_t i, const string &name)
	{
		matcolnames[i] = name;
	}

	/** Get the list of variables. */
	string getVarList() const
	{
		string res;
		size_t nvar = n_cols-1;
		for(size_t i=0;i<nvar;++i) {
			res.append(matcolnames[i]);
			if( i != nvar-1 )
				res.append(",");
		}
		return res;
	}

	/** Column position. */
	size_t getColPos(const string &name) const
	{
		vector<string>::const_iterator it =
			::find(matcolnames.begin(), matcolnames.end(), name);
		if( it == matcolnames.end() )
			return getNcols();
		return it - matcolnames.begin();
	}

	/** Row type. */
	RowType getRowType(size_t r) const
	{
		return matrowtype[r];
	}

	/** Is there a strict inequality? */
	bool isStrict() const
	{
		for (size_t i = 0 ; i < n_rows ; i++)
			if( matrowtype[i] == MAT_LT )
				return true;
		return false;
	}

	/** Get matrix value. */
	double get(size_t row, size_t col) const
	{
		return matcols[col][row];
	}

	/** Swap two columns. */
	void swap_col(size_t col1, size_t col2)
	{
		swap(matcols[col1], matcols[col2]);
		swap(matcolnames[col1], matcolnames[col2]);
	}

	/** Print the linear system. */
	friend ostream& operator <<(ostream &o, LinearSystem &m)
	{
		size_t end = m.n_cols-1;
		for (size_t j = 0 ; j < end ; j++)
			o << m.matcolnames[j] << "\t";
		o << "|\t" << m.matcolnames[end] << endl;
		for (size_t i = 0 ; i < m.n_rows ; i++) {
			for (size_t j = 0 ; j < end ; j++)
				o << m.get(i, j) << "\t";
				switch(m.matrowtype[i]) {
					case MAT_EQ:
						o << "=\t";
						break;
					case MAT_LE:
						o << "<=\t";
						break;
					case MAT_LT:
						o << "<\t";
						break;
					default:
						throw runtime_error("Unknown operator");
						break;
				}
				o << m.get(i, end) << endl;
		}
		return o;
	}

	/** Add a new column. */
	size_t add_column(const string &name)
	{
		return add_col(name.c_str());
	}

	/** Return a clone object. */
	LinearSystem *clone(bool content=true) const
	{
		LinearSystem *cl = new LinearSystem(0);
		cl->n_rows = content ? n_rows : 0;
		cl->n_cols = n_cols;
		(cl->matcols).reserve(n_cols);
		(cl->matcolnames).reserve(n_cols);
		if( content )
			(cl->matrowtype).reserve(n_rows);
		for(size_t i=0; i<n_cols; ++i) {
			double *col = new double[n_rows];
			(cl->matcols).push_back(col);
			(cl->matcolnames).push_back(matcolnames[i]);
			if( content )
				memcpy(col, matcols[i], n_rows*sizeof(double));
		}
		if( content )
			for(size_t i=0; i<n_rows; ++i)
				(cl->matrowtype).push_back(matrowtype[i]);
		return cl;
	}

	/** Transform the linear system into Polylib matrix.
	 *  Strict inequalities are transformed into non-strict ones. */
	Matrix * toPolylib()
	{
		size_t nrows = getNrows();
		size_t ncol = getNcols();
		Matrix *M = Matrix_Alloc(nrows, ncol+2);
		for(size_t i=0; i<nrows; ++i) {
			::Value *row = M->p[i];
			toPolylib(i, row);
		}
		return M;
	}
	
	/** Transform a row of the linear system into a row of the Polylib matrix.
	 *  Strict inequalities are transformed into non-strict ones. */
	void toPolylib(size_t i, ::Value *row)
	{
		size_t ncol = getNcols();
		row[0] = (getRowType(i)==LinearSystem::MAT_EQ) ? 0 : 1;
		double factor = 1;
		for(size_t j=0;j<=ncol;++j) {
			double dec = string_util::decimals( get(i, j) );
			if( dec > factor )
				factor = dec;
		}
		for(size_t j=1;j<=ncol;++j) {
			row[j] = -(::Value)(get(i,j-1)*factor);
		}
		row[ncol+1] = (::Value)(get(i,ncol)*factor);
	}
	
	/** True if system is homogeneous. */
	bool isHomo()
	{
		size_t nrows = getNrows();
		size_t ncol = getNcols();
		for(size_t i=0; i<nrows; ++i)
			if( get(i,ncol) )
				return false;
		return true;
	}

	/** Transform the homogeneous version of the linear system into Polylib matrix.
	 *  Strict inequalities are transformed into non-strict ones. */
	Matrix * toPolylibHomo()
	{
		size_t nrows = getNrows();
		size_t ncol = getNcols();
		Matrix *M = Matrix_Alloc(nrows, ncol+2);
		for(size_t i=0; i<nrows; ++i) {
			::Value *row = M->p[i];
			toPolylib(i, row);
			row[ncol+1] = 0;
		}
		return M;
	}
	
	void freeze()
	{
		frozen = true;
	}
	
	void unfreeze()
	{
		frozen = false;
	}

private:
	/** Number of rows */
	size_t n_rows;
	/** Number of columns */
	size_t n_cols;
	/** Matrix stored by columns. */
	vector<double *> matcols;
	/** Column names. */
	vector<string> matcolnames;
	/** Row types. */
	vector<RowType> matrowtype;
	/** No more variables? */
	bool frozen;

	/** Constructor. */
	LinearSystem(int _dummy)
		: n_rows(0), n_cols(0), frozen(false)
	{
	}

	/** Add a new column. */
	size_t add_col(const char *name = NULL)
	{
		if( frozen )
			throw runtime_error("Adding variable to frozen linear system");
		size_t space = sizeof(double)*n_rows;
		double *ptr =  new double[n_rows];
		memset(ptr, 0, space);
		matcolnames.push_back(name);
		matcols.push_back(ptr);
		if( n_cols > 0 )
			swap_col(n_cols-1, n_cols);
		n_cols++;
		return n_cols>1?n_cols-2:1;
	}

	/** Search and possibly add a column. */
	size_t search_add_col(const char *name)
	{
		vector<string>::const_iterator it =
			::find(matcolnames.begin(), matcolnames.end(), name);
		if( it == matcolnames.end() )
			return add_col(name);
		return it - matcolnames.begin();
	}

	/** Add new rows. */
	void add_rows(size_t newrows)
	{
		if( newrows == 0 ) return;
		// realloc existing columns;
		size_t totrow = n_rows + newrows;
		size_t old_space = sizeof(double)*n_rows;
		size_t new_space = sizeof(double)*newrows;
		for(size_t i=0;i<n_cols;++i) {
			double *ptr = new double[totrow];
			memcpy(ptr, matcols[i], old_space);
			memset(ptr+n_rows, 0, new_space);
			delete[] matcols[i];
			matcols[i] = ptr;
		}
		matrowtype.reserve(n_rows+newrows);
		for(size_t i=0; i<newrows; ++i)
			matrowtype.push_back(MAT_EQ); // default type
		n_rows += newrows;
	}

	/** Parse a primitive constraint. */
	void parse(const char *c, size_t row)
	{
		const char *end_token = string_util::find_sep3(c, '<', '=', '>');
		char *left = NULL;
		char *right = NULL;
		int factor = 1;
		if( *end_token == 0 )
			throw runtime_error("parse error in: "+string(c));
		if( (*end_token == '<') || (*end_token == '>') ) {
			if(*end_token == '>')
				factor = -1;
			left = string_util::strcpya(c, end_token-c);
			if( *(end_token+1) == '=' ) {
				matrowtype[row] = MAT_LE;
				right = string_util::strcpya(end_token+2);
			} else {
				matrowtype[row] = MAT_LT;
				right = string_util::strcpya(end_token+1);
			}
		} else {
			left = string_util::strcpya(c, end_token-c);
			if(  *(end_token+1) == '<' ) {
				right = string_util::strcpya(end_token+2);
				matrowtype[row] = MAT_LE;
			} else {
				right = string_util::strcpya(end_token+1);
				matrowtype[row] = MAT_EQ;
			}
		}
		parse_expr( left, row, factor);
		delete[] left;
		parse_expr( right, row, -factor);
		delete[] right;
	}

	/** Parse an expression. */
	void parse_expr(const char *c, size_t row, int factor)
	{
		int nextlocal = factor;
		while( *c ) {
			const char *end_token = string_util::find_sep2(c, '+', '-');
			if( (end_token > c) && ( *(end_token-1) == '*' ) )
				end_token = string_util::find_sep2(c+1, '+', '-');
			int local = nextlocal;
			nextlocal = ( *end_token == '-' ) ? -factor : factor;
			char *expr = string_util::strcpya(c, end_token-c);
			parse_mult(expr, row, local);
			delete[] expr;
			c = *end_token ? end_token+1 : end_token;
		}
	}

	/** Parse a multiplicative expression. */
	void parse_mult(const char *c, size_t row, int factor)
	{
		const char *end_token = string_util::find_sep(c, '*');
		if( *end_token == 0) {
			char *val = string_util::trim(c);
			double n = 0;
			if( string_util::is_number(val, &n) ) {
				size_t col = search_add_col(CONSTCOLNAME);
				matcols[col][row] += -factor*n;
				delete []val;
				return;
			}
			size_t col = search_add_col(val);
			matcols[col][row] += factor;
			delete []val;
			return;
		}

		char *left = string_util::strcpya(c, end_token-c);
		double n1 = 0, n2 = 0;
		size_t col = 0;
		char *val = string_util::trim(left);
		bool is1 = string_util::is_number(val, &n1);
		if( !is1 )
			col = search_add_col(val);
		delete []val;
		val = string_util::trim(end_token+1);
		bool is2 = string_util::is_number(val, &n2);
		if( !is1 && !is2 )
			throw runtime_error("parse error in: "+string(c));
		if( is1 && is2 ) {
			size_t col = search_add_col(CONSTCOLNAME);
			matcols[col][row] += -n1*n2*factor;
			delete []val;
			return;
		}
		if( !is2 )
			col = search_add_col(val);
		matcols[col][row] += (is1?n1:n2)*factor;
		delete []val;
	}
};


/**
* Utility methods for (parameterized) polyhedra.
*/
class poly_util
{
public:
	/** Compute inner product of two vectors. */
	static ::Value inner(const ::Value *r1, const ::Value *r2, size_t n)
	{
		const ::Value *end = r1+n;
		::Value r = 0;
		while( r1 < end )
			r += (*r1++) * (*r2++);
		return r;
	}

	/** Check if c x = alpha over a polyhedron. */
	static bool expr_equal(const Polyhedron *pol, const ::Value *row)
	{
		// row[1], .., row[d] variable coefficients
		// row[d+1] alpha
		size_t Dimension = pol->Dimension;
		size_t NbRays = pol->NbRays;
		::Value alpha = row[Dimension+1];
//		Polyhedron_Print(stdout,P_VALUE_FMT,pol);
//		for(size_t i=0;i<=Dimension+1;++i)
//			cout << "row["<<i<<"] = " << row[i] << endl;
		for (size_t i=0;i<NbRays;++i) {
			::Value *p=pol->Ray[i];
			if( value_notzero_p(*p) ) {
				::Value pd1 = p[Dimension+1];
				if(value_notzero_p(pd1) ) {
					 // vertex
					::Value ip = inner(row+1, p+1, Dimension);
					if( ip != alpha*pd1 )
						return false;
					continue;
				}
				// ray
				::Value ip = inner(row+1, p+1, Dimension);
				if( ip != 0 )
					return false;
			} else {
				// line
				::Value ip = inner(row+1, p+1, Dimension);
				if( ip != 0 )
					return false;
			}
		}
		return true;
	}

	/** Swap dimensions in a polyhedron. */
	static void swap_pol_dim(Polyhedron *Pol, size_t d1, size_t d2)
	{
		if( d1 == d2 )
			return;
		size_t NbRays = Pol->NbRays;
		++d1;
		++d2;
		for (size_t i=0;i<NbRays;++i) {
			::Value *p=Pol->Ray[i];
			swap(p[d1], p[d2]);
		}

		size_t NbConstraints = Pol->NbConstraints;
		for (size_t i=0;i<NbConstraints;++i) {
			::Value *p=Pol->Constraint[i];
			swap(p[d1], p[d2]);
		}
	}

	/** Compare two rows. */
	inline static bool row_equal(::Value *r1, ::Value *r2, size_t d)
	{
		::Value *end = r1+d;
		while( r1 < end )
			if( *r1++ != *r2++ )
				return false;
		return true;
	}

	/** Returns the variability range of a variable in a polyhedron.  Returns -1 if range is infinity or greater than maxr. */
	static double range_width(Polyhedron *Pol, size_t idx, double rmax = -1)
	{
		size_t Dimension = Pol->Dimension;
		size_t NbRays = Pol->NbRays;
		bool firstvertex = true;
		double minv = 0, maxv = 0;
		bool checkmax = rmax >= 0;
		for (size_t i=0;i<NbRays;++i) {
			::Value *p=Pol->Ray[i];
			if( value_notzero_p(*p) ) {
				::Value pd1 = p[Dimension+1];
				if(value_notzero_p(pd1) ) {
					// vertex
					double value = p[idx+1]/(double)pd1;
					if( firstvertex ) {
						minv = maxv = value;
						firstvertex = false;
						continue;
					}
					if( value > maxv ) {
						maxv = value;
						if( (checkmax) && (maxv-minv > rmax) )
							return -1;
					}
					else if( value < minv ) {
						minv = value;
						if( (checkmax) && (maxv-minv > rmax) )
							return -1;
					}
					continue;
				}
				// ray
				if( p[idx+1] != 0 )
					return -1;
			} else {
				// line
				if( p[idx+1] != 0 )
					return -1;
			}
		}
		return maxv-minv;
	}

	/** Returns the largest r such that c x + alpha =_r 0  over a polyhedron with r <= rmax.  Returns -1 if there is some r > rmax. */
	static double expr_range_width(const Polyhedron *pol, const ::Value *row, double rmax = -1)
	{
		// row[0], .., row[d-1] variable coefficients
		// row[d] alpha
		// row[d+1] divisor
		size_t Dimension = pol->Dimension;
		size_t NbRays = pol->NbRays;
		double alpha = row[Dimension];
		double divisor = row[Dimension+1];
		double maxr=0;
		bool firstvertex=true;
		for (size_t i=0;i<NbRays;++i) {
			::Value *p=pol->Ray[i];
			if( value_notzero_p(*p) ) {
				::Value pd1 = p[Dimension+1];
				if(value_notzero_p(pd1) ) {
					 // vertex
					 double r = (((double)inner(row, p+1, Dimension))/pd1 - alpha)/divisor;
					 if( r < 0 ) r = -r;
					 if( firstvertex ) {
						 firstvertex = false;
						 maxr = r;
						 continue;
					 }
					 if( rmax >= 0 && (r > rmax) )
						return -1;
					 if( r > maxr )
						maxr = r;
					 continue;
				}
				// ray
				::Value ip = poly_util::inner(row, p+1, Dimension);
				if( ip != 0 )
					return -1;
			} else {
				// line
				::Value ip = poly_util::inner(row, p+1, Dimension);
				if( ip != 0 )
					return -1;
			}
		}
		return maxr;
	}

	/** Infer the largest r such that c_1 x + alpha_1 =_r c_2 x + alpha_2 over a polyhedron with r <= rmax.  Returns -1 if there is some r > rmax. */
	static double expr_range_width(const Polyhedron *pol, const ::Value *r1, const ::Value *r2, double rmax = -1)
	{
		// r1[0], .., r1[d-1] variable coefficients
		// r1[d] alpha
		// r1[d+1] divisor
		size_t Dimension = pol->Dimension;
		::Value row[Dimension+2];
		size_t a = r1[Dimension+1], b = r2[Dimension+1];
		// BUG: [efficiency] lcm between a and b should be set instead of a*b
		for(size_t i=0; i<Dimension; ++i)
			row[i] = r1[i]*b - r2[i]*a;
		row[Dimension] = r2[Dimension]*a - r1[Dimension]*b;
		row[Dimension+1] = a * b;
		return expr_range_width(pol, row, rmax);
	}

#ifndef DOXYGEN_SHOULD_SKIP_THIS
	/** List of polyhedra with a same parameter expression. */
	class distinctRowDomain
	{
		public:
			/** an expression over parameters */
			::Value *row;
			/** domains for the expression */
			vector<Polyhedron *> pdom;
	};
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

	/** Infer the variability range of a variable in a parameterized polyhedron. */
	static double range_width(Param_Vertices *PV,Param_Domain *PD, size_t idx, double rmax = -1)
	{
		// BUG: [efficiency] better to implement as a map
		vector<distinctRowDomain> dist;
		dist.reserve(8);

		Param_Vertices *pv1=PV;
		size_t nc = pv1->Vertex->NbColumns;
		// scan vertices and domains
		double range = 0;
		for (Param_Domain *p1=PD; p1; p1=p1->next, pv1=pv1->next)
		{
			::Value **pve1 = pv1->Vertex->p; // vertex
			::Value *row = pve1[idx]; // parameterized expression for variable idx
			 // search for row in dist
			 size_t distn = dist.size();
			 size_t foundexpr = distn;
			 bool founddom = false;
			 for(size_t d = 0; d < distn; ++d)
				 if( poly_util::row_equal(dist[d].row, row, nc) ) {
					foundexpr = d;
					vector<Polyhedron *> &pd = dist[d].pdom;
					size_t ndom = pd.size();
					for(size_t i =0; i < ndom; ++i) {
						if( PolyhedronIncludes(pd[i], p1->Domain) ) {
							founddom = true;
							break;
						}
					}
					if( !founddom)
						pd.push_back( p1->Domain );
					break;
				 }
			if( founddom )
				continue; // expr and domain already checked
			if( foundexpr == distn ) {
				// not found, add it
				dist.push_back(distinctRowDomain());
				dist[distn].row = row;
				dist[distn].pdom.reserve(8);
				dist[distn].pdom.push_back( p1->Domain );
			}
			// compare with every distinct
			for(size_t d = 0; d < distn; ++d) {
				if( d == foundexpr )
					continue;
				// compare with every polyhedron
				vector<Polyhedron *> &pd = dist[d].pdom;
				size_t ndom = pd.size();
				for(size_t i =0; i < ndom; ++i) {
					// compute intersection
					Polyhedron *dx = PDomainIntersection(p1->Domain,pd[i],0);
					// if not empty
					if(dx && !emptyQ(dx)) {
						// if equal over the intersection
						double r = poly_util::expr_range_width(dx, row, dist[d].row, rmax);
						if( r < 0 ) {
							Domain_Free(dx);
							return -1;
						}
						if( r > range )
							range = r;
					}
					if (dx != NULL)
						Domain_Free(dx);
				}
			}
		}
		return range;
    }

	/** Check the variability range of a parametric typed variable in a parameterized polyhedron. */
	static bool range_width_check(Param_Vertices *PV, Param_Domain *PD, size_t idx, const ::Value *rowe)
	{
		// BUG: [efficiency] better to implement as a map
	    vector< poly_util::distinctRowDomain > dist;
	    dist.reserve(8);

		Param_Vertices *pv1=PV;
		size_t nc = pv1->Vertex->NbColumns;
		// scan vertices and domains
	    for (Param_Domain *p1=PD; p1; p1=p1->next, pv1=pv1->next)
	    {
			::Value **pve1 = pv1->Vertex->p; // vertex
			::Value *row = pve1[idx]; // parameterized expression for variable idx
			
			// check if e >= 0 over vertex domain
			if( !expr_gt(p1->Domain, rowe) ) {
//				cout << "parametric expression negative" << endl;
				return false;
			}
				
			// search for row in dist
			size_t distn = dist.size();
			size_t foundexpr = distn;
			bool founddom = false;

			for(size_t d = 0; d < distn; ++d)
				if( poly_util::row_equal(dist[d].row, row, nc) ) {
					foundexpr = d;
					vector<Polyhedron *> &pd = dist[d].pdom;
					size_t ndom = pd.size();
					for(size_t i =0; i < ndom; ++i) {
						if( PolyhedronIncludes(pd[i], p1->Domain) ) {
							founddom = true;
							break;
						}
					}
					if( !founddom)
						pd.push_back( p1->Domain );
					break;
				 }
			if( founddom )
				continue; // expr and domain already checked
			if( foundexpr == distn ) {
				// not found, add it
				dist.push_back(poly_util::distinctRowDomain());
				dist[distn].row = row;
				dist[distn].pdom.reserve(8);
				dist[distn].pdom.push_back( p1->Domain );
			}
			// compare with every distinct
			for(size_t d = 0; d < distn; ++d) {
				if( d == foundexpr )
					continue;
				// compare with every polyhedron
				vector<Polyhedron *> &pd = dist[d].pdom;
				size_t ndom = pd.size();
				for(size_t i =0; i < ndom; ++i) {
					// compute intersection
					Polyhedron *dx = PDomainIntersection(p1->Domain,pd[i],0);
					// if not empty
					if(dx && !emptyQ(dx)) {
						// if equal over the intersection
						bool r = expr_lt_r(dx, row, dist[d].row, rowe);
						if( !r ) {
							Domain_Free(dx);
							return false;
						}
					}
					if (dx != NULL)
						Domain_Free(dx);
				}
			}
		}
	   return true;
	}

	/** Check if c_1 x + alpha_1 <=_e c_2 x + alpha_2 over a polyhedron. */
	static bool expr_lt_r(const Polyhedron *pol, const ::Value *r1, const ::Value *r2, const ::Value *re)
	{
		// r1[0], .., r1[d-1] variable coefficients
		// r1[d] alpha
		// r1[d+1] divisor

		size_t Dimension = pol->Dimension;
		::Value row[Dimension+2];
		::Value row2[Dimension+2];

		size_t a = r1[Dimension+1], b = r2[Dimension+1], c = re[Dimension+1];
		// BUG: [efficiency] lcm(a,b,c) should be set instead of a*b*c
		size_t bc = b*c, ac = a*c, ab = a*b;
		for(size_t i=0; i<Dimension; ++i) {
			::Value val = r1[i]*bc - r2[i]*ac;
			::Value vale = re[i]*ab;
			row[i] = val - vale;
			row2[i] = - val - vale;

//			cout << "i=" << i << " r1[i]=" << r1[i] << " r2[i]=" << r2[i]
//				<< " re[i]=" << re[i] << " row[i]=" <<row[i] << " row2[i]=" << row2[i] << endl;
		}
		::Value val = r1[Dimension]*bc - r2[Dimension]*ac;
		::Value vale = re[Dimension]*ab;
		row[Dimension] = - val + vale;
		row2[Dimension] = val + vale;
//		cout << "D=" << Dimension << " r1[D]=" << r1[Dimension] << " r2[D]=" << r2[Dimension]
//			<< " re[D]=" << re[Dimension] << " row[D]=" <<row[Dimension] << " row2[D]=" << row2[Dimension] << endl;
// divisor not needed
//			row[Dimension+1] =
//			row2[Dimension+1] = a*bc;
		double alpha = row[Dimension];
		double alpha2 = row2[Dimension];

		size_t NbRays = pol->NbRays;
		for (size_t i=0;i<NbRays;++i) {
			::Value *p=pol->Ray[i];
			if( value_notzero_p(*p) ) {
				::Value pd1 = p[Dimension+1];
				if(value_notzero_p(pd1) ) {
					 // vertex
					 double r = (((double)poly_util::inner(row, p+1, Dimension))/pd1 - alpha); // /row[Dimension+1]  not necessary
					 if( r > 0 )
						return false;
					 double r2 = (((double)poly_util::inner(row2, p+1, Dimension))/pd1 - alpha2); // /row2[Dimension+1]  not necessary
					 if( r2 > 0 )
						return false;
					 continue;
				}
				// ray
				::Value ip = poly_util::inner(row, p+1, Dimension);
				if( ip > 0 )
					return false;
				::Value ip2 = poly_util::inner(row2, p+1, Dimension);
				if( ip2 > 0 )
					return false;
			} else {
				// line
				::Value ip = poly_util::inner(row, p+1, Dimension);
				if( ip != 0 )
					return false;
				::Value ip2 = poly_util::inner(row2, p+1, Dimension);
				if( ip2 != 0 )
					return false;
			}
		}
		return true;
	}

	/** Check if c x + alpha_1 >= 0 over a polyhedron. */
	static bool expr_gt(const Polyhedron *pol, const ::Value *row)
	{
		// row[0], ..,re[d-1] variable coefficients
		// row[d] alpha
		// rowd+1] divisor
		size_t Dimension = pol->Dimension;
		double alpha = row[Dimension];
		//for(size_t i=0; i<Dimension+2; ++i)
		//	cout << row[i] << " ";
		//cout << endl;

		size_t NbRays = pol->NbRays;
		for (size_t i=0;i<NbRays;++i) {
			::Value *p=pol->Ray[i];
			if( value_notzero_p(*p) ) {
				::Value pd1 = p[Dimension+1];
				if(value_notzero_p(pd1) ) {
					 // vertex
					 double r = (((double)poly_util::inner(row, p+1, Dimension))/pd1 + alpha); // /row[Dimension+1]  not necessary
					 if( r < 0 )
						return false;
					 continue;
				}
				// ray
				::Value ip = poly_util::inner(row, p+1, Dimension);
				if( ip < 0 )
					return false;
			} else {
				// line
				::Value ip = poly_util::inner(row, p+1, Dimension);
				if( ip != 0 )
					return false;
			}
		}
		return true;
	}
};

} // end of namespace util

using namespace util;

#endif // !defined(_UTIL_H__INCLUDED_)
